/**************************************************************/
/*                                                            */
/*                                                            */
/*  Written by Jim MacNair - October, 1997                    */
/*                                                            */
/*                                                            */
/**************************************************************/

/**************************************************************/
/* Required includes for C functions                          */
/**************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <locale.h>

/**************************************************************/
/* Required include for CICS ECI calls                        */
/**************************************************************/

#define CICS_W32
#include <cics_eci.h>

/**************************************************************/
/* Required include for Common subroutines                    */
/**************************************************************/

#include "comsubs.h"

/**************************************************************/
/* Required include for CICS ECI calls                        */
/**************************************************************/

#include "ecimsg.h"

/**************************************************************/
/* Routine to print CICS ECI error messages                   */
/**************************************************************/

char* Print_ECI_Error(int rc)
{
	char *msgptr;

        switch (rc)
         {
          case ECI_ERR_INVALID_DATA_LENGTH:
            msgptr = &"Invalid Data Length";
            break;

          case ECI_ERR_INVALID_EXTEND_MODE:
            msgptr = &"Invalid Extend Mode";
            break;

          case ECI_ERR_NO_CICS:
            msgptr = &"CICS is not running";
            break;

          case ECI_ERR_CICS_DIED:
            msgptr = &"CICS terminated unexpectedly";
            break;

          case ECI_ERR_REQUEST_TIMEOUT:
            msgptr = &"Request timed out";
            break;

          case ECI_ERR_RESPONSE_TIMEOUT:
            msgptr = &"Response timed out";
            break;

          case ECI_ERR_TRANSACTION_ABEND:
            msgptr = &"Transaction abended";
            break;

          case ECI_ERR_EXEC_NOT_RESIDENT:
            msgptr = &"Not resident";
            break;

          case ECI_ERR_SYSTEM_ERROR:
            msgptr = &"System error";
            break;

          case ECI_ERR_NULL_WIN_HANDLE:
            msgptr = &"Window handle cannot be null";
            break;

          case ECI_ERR_NULL_MESSAGE_ID:
            msgptr = &"Invalid message id";
            break;

          case ECI_ERR_THREAD_CREATE_ERROR:
            msgptr = &"Unable to create thread";
            break;

          case ECI_ERR_INVALID_CALL_TYPE:
            msgptr = &"Unknown call type";
            break;

          case ECI_ERR_ALREADY_ACTIVE:
            msgptr = &"Another request is already active";
            break;

          case ECI_ERR_RESOURCE_SHORTAGE:
            msgptr = &"A resource shortage occured";
            break;

          case ECI_ERR_NO_SESSIONS:
            msgptr = &"No sessions are available";
            break;

          case ECI_ERR_NULL_SEM_HANDLE:
            msgptr = &"The semaphore handle cannot be null";
            break;

          case ECI_ERR_INVALID_DATA_AREA:
            msgptr = &"Invalid data area pointer";
            break;

          case ECI_ERR_INVALID_VERSION:
            msgptr = &"Invalid version number";
            break;

          case ECI_ERR_UNKNOWN_SERVER:
            msgptr = &"Cannot find the named CICS server";
            break;

          case ECI_ERR_CALL_FROM_CALLBACK:
            msgptr = &"Callback routine cannot use ECI calls";
            break;

          case ECI_ERR_INVALID_TRANSID:
            msgptr = &"Transaction not found on CICS server";
            break;

          case ECI_ERR_MORE_SYSTEMS:
            msgptr = &"More systems error";
            break;

          case ECI_ERR_NO_SYSTEMS:
            msgptr = &"No systems are available";
            break;

          case ECI_ERR_SECURITY_ERROR:
            msgptr = &"Security error";
            break;

          case ECI_ERR_MAX_SYSTEMS:
            msgptr = &"Max systems exceeded";
            break;

          case ECI_ERR_MAX_SESSIONS:
            msgptr = &"Max sessions exceeded";
            break;

          case ECI_ERR_ROLLEDBACK:
            msgptr = &"CICS transaction was rolled back unexpectedly";
            break;

          default:
            msgptr = &"Unknown error";
            break;
         }

	return msgptr;
}

/**************************************************************/
/* Routine to execute a CICS ECI call                         */
/* The token parameter is only used for extended calls and    */
/* commit calls.  It should be initialized to zero for the    */
/* first call of an extended call sequence, and should be     */
/* passed back unchanged on all subsequent extended calls.    */                                 
/**************************************************************/

int doECI(const char *pgmName, const char *userid, const char *pw, 
		  const char *sysid, char *pCommArea, int commLen, VARSTRUCT *pVars, 
		  char *abendcode, int calltype, int *token)
{
    ECI_PARMS   eciParms;
	int			rc;
	char		*msgptr;
	char		vardata[MAX_DATA_LEN];
	const char	*currErr;

/**************************************************************/
/* Initialize variables                                       */
/**************************************************************/

	memset(abendcode, '\0', ABCODE_LEN + 1);

/**************************************************************/
/* Build the ECI block                                        */
/**************************************************************/

    memset(&eciParms,'\0',sizeof(eciParms));
    setlocale(LC_ALL,"");
    eciParms.eci_call_type = ECI_SYNC;
    eciParms.eci_version = ECI_VERSION_1;
    eciParms.eci_luw_token = *token;
	eciParms.eci_extend_mode = calltype;

	/* Set the user id and password */
	if (userid != NULL)
	{
		memcpy(eciParms.eci_userid,userid,MAX_PW_LEN);
	}

	if (pw != NULL)
	{
		memcpy(eciParms.eci_password,pw,MAX_PW_LEN);
	}

	/* Set up the CICS commarea */
    eciParms.eci_commarea = pCommArea;
    eciParms.eci_commarea_length = commLen;

	if ((sysid != NULL) && (sysid[0] != '\0'))
	{
		memcpy(eciParms.eci_system_name,sysid,MAX_SYS_NAME_LEN);
	}

	/* Set the program name */
	if (pgmName != NULL)
	{
		memcpy(eciParms.eci_program_name,pgmName,MAX_PROG_NAME_LEN);
	}

	/* Execute the request */
    rc = CICS_ExternalCall(&eciParms);

	/* Check if it worked */
    if (rc == ECI_NO_ERROR)
	{
		/* return the token, if any, to the calling program */
		*token = eciParms.eci_luw_token;
	}
	else
    {
		/* Check for specific types of error codes with additional useful info */
		/* Create a variable with the ECI error message */
		msgptr = Print_ECI_Error(rc);

		/* check if there is already an ECI error message   */
		/* If there is, then we do not want to overwrite it */
		currErr = GetVarPtr(ECI_ERR_MSG, pVars);
		if (currErr == NULL)
		{
			AddVar(ECI_ERR_MSG, msgptr, pVars);

			/* Get the ECI error code in a message */
		    sprintf(vardata, " ECI Return Code %d", rc);
			AddVar(ECI_ERR_CODE, vardata, pVars);

			/* Create Ending and Beginning Text */
			AddVar(ECI_ERR_HR, ECI_HR, pVars);
			AddVar(ECI_ERR_NL, ECI_NEWLINE, pVars);

	        if (rc == ECI_ERR_SYSTEM_ERROR)
			{
				/* Get the system error if it exists */
				sprintf(vardata, " System Error Code %d", eciParms.eci_sys_return_code);
				AddVar(ECI_ERR_SYSERR, vardata, pVars);
			}

			if ((rc == ECI_ERR_TRANSACTION_ABEND) && (eciParms.eci_abend_code != 0))
			{
				/* Get the abend code if it exists */
	            memcpy(abendcode, eciParms.eci_abend_code, ABCODE_LEN);
		        sprintf(vardata, " CICS Abend Code %s", abendcode);
				AddVar(ECI_ERR_ABCODE, vardata, pVars);
			}
		}
    }

	return rc;
}

/**************************************************************/
/* Routine to perform single ECI calls                        */
/**************************************************************/

int performECI(const char *pgmName, const char *userid, const char *pw, 
			   const char *sysid, char *pCommArea, int commLen, VARSTRUCT *pVars, 
			   char *abendcode, int *token)

{
	int rc = 0;

	/* Perform the actual ECI call */
	rc = doECI(pgmName, userid, pw, sysid, pCommArea, commLen, pVars, 
			   abendcode, ECI_NO_EXTEND, token);

	return rc;
}

/**************************************************************/
/* Routine to perform extended ECI calls.                     */
/* Extended ECI calls have more than one ECI call in the      */
/* same unit of work.                                         */
/**************************************************************/

int perfECIext(const char *pgmName, const char *userid, const char *pw, 
			   const char *sysid, char *pCommArea, int commLen, VARSTRUCT *pVars, 
			   char *abendcode, int *token)

{
	int rc = 0;

	/* Perform the actual ECI call */
	rc = doECI(pgmName, userid, pw, sysid, pCommArea, commLen, pVars, 
			   abendcode, ECI_EXTENDED, token);

	return rc;
}

/**************************************************************/
/* Routine to end an extended ECI call without executing      */
/* a CICS program.                                            */
/**************************************************************/

int perfCommit(const char *pgmName, const char *userid, const char *pw, 
			   const char *sysid, char *pCommArea, int commLen, VARSTRUCT *pVars, 
			   char *abendcode, int *token)

{
	int rc = 0;

	/* Perform the actual ECI call */
	rc = doECI(pgmName, userid, pw, sysid, pCommArea, commLen, pVars, 
			   abendcode, ECI_COMMIT, token);

	return rc;
}

void setStandardHeader(pCOMMHEAD ca, 
					   long calen, 
					   int maplen, 
					   char *mapname, 
					   char *tranname,
 					   char *cics_client_name)

{
	char *map = "";
	char *tran = "";


	// Get mapname and tranname into temporary CStrings
	if (mapname != NULL)
	{
		map = mapname;
	}

	if (tranname != NULL)
	{
		tran = tranname;
	}

	// set the version number to zero
	memset(ca->VERSION, '0', sizeof(ca->VERSION));

	// Set the Map Name field in the COMMAREA
	char_to_comm(map, ca->MAPNAME, sizeof(ca->MAPNAME), ALPHA_FIELD);

	// Set the User Transaction Name field in the COMMAREA
	char_to_comm(tran, ca->USRTRNID, sizeof(ca->USRTRNID), ALPHA_FIELD);

	// Set the Client Name field in the COMMAREA
	char_to_comm(cics_client_name, ca->USRTRMID, sizeof(ca->USRTRMID), ALPHA_FIELD);

	// set the user commarea length 
	int_to_comm(calen, ca->USRCALEN, sizeof(ca->USRCALEN));

	// set the BMS data length 
	int_to_comm(maplen, ca->USRMAPLN,sizeof(ca->USRMAPLN));

	// Initialize the return code and next transaction id
	memset(ca->RETCODE, '0', sizeof(ca->RETCODE));
	memset(ca->NXTRNID, ' ', sizeof(ca->NXTRNID));

	// Set default aid byte to enter key
	ca->USRAID = '\x027';

	// Set default cursor position to 0,0
	memset(ca->USRCPOS, '0', sizeof(ca->USRCPOS));
}

/****************/
/* End of File  */
/****************/