// statfile.cpp: implementation of the statfile class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "statfile.h"
#include "comsubs.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

////////////////////////////////////////////////////////////////
// Status file header structure definition                        
////////////////////////////////////////////////////////////////

// CICS Status file header structure
typedef struct {
   char userName[8];
   char passWord[8];
   char sysName[8];
   char clientName[4];
   char replayCount[8];
   char tctuaLength[4];
   char caLength[8];
} STATUS_FILE_HEADER;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Cstatfile::Cstatfile()
{

}

Cstatfile::~Cstatfile()
{

}

///////////////////////////////////////////////////////////////
//
// This routine will save the current CICS status in a 
// temporary file.  The temporary file name is generated
// when the user logs on, and is saved in all subsequent
// web pages as a hidden variable.  It could also be saved
// as a cookie.  The replay count is used to detect 
// replays of a previous HTML page.  It will be automatically
// updated before the temporary file is written.
//
///////////////////////////////////////////////////////////////

int	Cstatfile::updateStatusfile(Cvartable& vars, 
								const int tctualen,
								const char* tctua,
								const int calen, 
								const char* ca)

{
	int		rc=0;			// return code variable
	FILE	*statusfile;	// file handle for the temporary file
	STATUS_FILE_HEADER	hdrData;	// Status file header data
	CString	user;
	CString	pw;
	CString	client;
	CString	sys;
	char	*replaycount;
	char	*statusfilename;	// Name of Status File
	char	temptctua[256];
	int		bytesread;
	int		temptctlen=0;
	int		replay=0;

	// Get the status file name from the HTML variables
	statusfilename  = (char *) vars.GetVarPtr(STATFILE);

	// Trace point for debugging
	ISAPITRACE("Entering updateStatusfile %s\n", statusfilename);

	// Validate the input parameters
	// Make sure the areas are within CICS limits
	ISAPIASSERT(tctualen < 256);	// Stay within CICS limits
	ISAPIASSERT(calen <= 32767);	// Stay within CICS limits

	// Clear the initial TCTUA area
	memset(temptctua, 0, sizeof(temptctua));

	// First, read the contents of the current file, to
	// retrieve the tctua length and any current tctua data

	// Open the status file for input
	statusfile = fopen(statusfilename, "rb");

	// Check if it worked
	if (statusfile == NULL)
	{
		// Trace point for debugging
		ISAPITRACE("File open failed for file %s\n", statusfilename);

		// Didn't work, set return code to 1
		return 1;
	}
	else
	{
		// file is open, read the first part of the file
		bytesread = fread(&hdrData, 1, sizeof(hdrData), statusfile);

		// Make sure that we read the entire header
		if (bytesread != sizeof(hdrData))
		{
			return 99;
		}

		// Set the TCT user area length field
		temptctlen = CComsubs::comm_to_int(hdrData.tctuaLength, sizeof(hdrData.tctuaLength));

		// Check if the TCTUA length in the status file is greater than zero
		if (temptctlen > 0)
		{
			// read the TCTUA from the status file
			temptctlen = fread(temptctua, 1, temptctlen, statusfile);
		}

		// Close the file
		fclose(statusfile);
	}

	// Next, move any updated TCT data that is being passed
	if (tctualen > 0)
	{
		// Get the updated data into the temporary TCTUA area
		memcpy(temptctua, tctua, tctualen);

		// Check if this application has increased the size of
		// the TCTUA
		if (tctualen > temptctlen)
		{
			// Yes, use the larger size when we rewrite the status file
			temptctlen = tctualen;
		}
	}

	// open the file for output, erasing any previous files
	statusfile = fopen(statusfilename, "wb");

	// Check if it worked
	if (statusfile == NULL)
	{
		// Didn't work, set return code to 1
		rc = 1;
	}
	else
	{
		//Capture the current CICS status from the variable table
		user   = vars.GetVarPtr("CICSUSER");
		pw     = vars.GetVarPtr("CICSPW");
		sys    = vars.GetVarPtr("CICSSYS");
		client = vars.GetVarPtr("CICSCLIENT");

		// Get the replay count
		replaycount = (char *) vars.GetVarPtr("REPLAYCOUNT");

		// convert the replay counter to an integer
		replay = atoi(replaycount);

		// Increment the replay counter
		replay++;

		// file is open, build the first part of the file
		// Get the user id, padded with spaces to 8 characters
		CComsubs::to_comm(user, hdrData.userName, sizeof(hdrData.userName), ALPHA_FIELD); 

		// Get the password, padded with spaces to 8 characters
		CComsubs::to_comm(pw, hdrData.passWord, sizeof(hdrData.passWord), ALPHA_FIELD); 

		// Get the system name, padded with spaces to 8 characters
		CComsubs::to_comm(sys, hdrData.sysName, sizeof(hdrData.sysName), ALPHA_FIELD); 

		// Get the client name, padded with spaces to 4 characters
		CComsubs::to_comm(client, hdrData.clientName, sizeof(hdrData.clientName), ALPHA_FIELD); 

		// Set the replay count
		CComsubs::int_to_comm(replay, hdrData.replayCount, sizeof(hdrData.replayCount));

		// Set the TCT user area length field
		CComsubs::int_to_comm(temptctlen, hdrData.tctuaLength, sizeof(hdrData.tctuaLength));

		// Set the COMMAREA length field
		CComsubs::int_to_comm(calen, hdrData.caLength, sizeof(hdrData.caLength));

		// Write the system name to the file
		fwrite(&hdrData, 1, sizeof(hdrData), statusfile);

		// Check if there is a TCT user area to save
		if (temptctlen > 0)
		{
			fwrite(temptctua, 1, temptctlen, statusfile);
		}

		// Check if there is a CICS user COMMAREA to save
		if ((ca != NULL) && (calen > 0))
		{
			fwrite(ca, 1, calen, statusfile);
		}

		// Close the file
		fclose(statusfile);
	}

	return rc;
}

///////////////////////////////////////////////////////////////
//
// This routine will save the current CICS status in a 
// temporary file.  The temporary file name is generated
// when the user logs on, and is saved in all subsequent
// web pages as a hidden variable.  It could also be saved
// as a cookie.  The replay count is used to detect 
// replays of a previous HTML page.  It will be automatically
// updated before the temporary file is written.
//
// 
//
///////////////////////////////////////////////////////////////

int	Cstatfile::getStatusfileData(Cvartable& vars, 
								 int* replay,
								 int* tctualen,
								 char* tctua,
								 int* calen, 
								 char* ca)

{
	int		rc=0;			// return code variable
	int		bytesread=0;	// number of bytes read
	FILE	*statusfile;	// file handle for the temporary file
	char	*statusfilename;
	STATUS_FILE_HEADER	hdrData;	// Status file header data
	CString	user;
	CString	pw;
	CString	sys;
	CString	client;
	CString replaycnt;
	char	temptctua[256];
	int		temptctlen;

	// Trace point for debugging
	ISAPITRACE("Entering getStatusfileData\n");

	// Validate the input parameters
	// Make sure the areas are within CICS limits
	ISAPIASSERT(*tctualen < 256);	// Stay within CICS limits
	ISAPIASSERT(*calen <= 32767);	// Stay within CICS limits

	// Clear the initial TCTUA area
	memset(temptctua, 0, sizeof(temptctua));

	// open the file for input
	statusfilename = (char *) vars.GetVarPtr(STATFILE);

	// Open the status file for input
	statusfile = fopen(statusfilename, "rb");

	// Check if it worked
	if (statusfile == NULL)
	{
		// Didn't work, set return code to 1
		rc = 1;
		// Trace point for debugging
		ISAPITRACE("File open failed for file %s\n", statusfilename);
	}
	else
	{
		// file is open, read the first part of the file
		bytesread = fread(&hdrData, 1, sizeof(hdrData), statusfile);

		if (bytesread == sizeof(hdrData))
		{
			// Extract the individual fields from the header
			CComsubs::from_comm(user, hdrData.userName, sizeof(hdrData.userName));
			CComsubs::from_comm(pw, hdrData.passWord, sizeof(hdrData.passWord));
			CComsubs::from_comm(sys, hdrData.sysName, sizeof(hdrData.sysName));
			CComsubs::from_comm(client, hdrData.clientName, sizeof(hdrData.clientName));
			CComsubs::from_comm(replaycnt, hdrData.replayCount, sizeof(hdrData.replayCount));
			*replay = CComsubs::comm_to_int((hdrData.replayCount), sizeof(hdrData.replayCount));

			// Capture the CICS status fields
			vars.AddVar("CICSUSER", user);
			vars.AddVar("CICSPW", pw);
			vars.AddVar("CICSSYS", sys);
			vars.AddVar("CICSCLIENT", client);
			vars.AddVar("REPLAYCOUNT", replaycnt);

			// Get the length of the TCTUA in the status file
			temptctlen = CComsubs::comm_to_int(hdrData.tctuaLength, sizeof(hdrData.tctuaLength));

			/////////////////////////////////////////////////////////////////////
			// Note that the TCTUA will be read into a temporary area first
			// This allows for the case where there is a TCTUA but the 
			// application does not use it and does not allocate an area for
			// the TCTUA.  Whatever length is requested will be returned.  If
			// the requested length is shorter than the actual TCTUA, only 
			// the requested number of characters will be returned.  If the
			// number is greater than the current TCTUA length in the status
			// file, the end of the TCTUA will be padded with binary zeros.
			// If present, the TCTUA on the status file must be read even if
			// the application does not use it, to get to the user COMMAREA.
			/////////////////////////////////////////////////////////////////////

			// Check if the TCTUA length in the status file is greater than zero
			if (temptctlen > 0)
			{
				// read the TCTUA from the status file
				temptctlen = fread(temptctua, 1, temptctlen, statusfile);
			}

			// Check if the application has allocated a TCTUA
			// If it has, pass back the number of characters requested
			if ((tctua != NULL) && (*tctualen > 0))
			{
				// pass back the requested bytes
				memcpy(tctua, temptctua, *tctualen);
			}
			else
			{
				// The application has no TCTUA, pass back zero bytes
				*tctualen = 0;
			}

			/////////////////////////////////////////////////////////////////////
			// The user COMMAREA is the last thing in the file.  It is replaced
			// after each transaction.  Therefore, it can be read directly
			// into the provided area.  If no area is allocated, then any 
			// user COMMAREA in the status file will be ignored.
			/////////////////////////////////////////////////////////////////////

			// Check if the user COMMAREA length is greater than zero
			if ((*calen > 0) && (ca != NULL))
			{
				// read the user COMMAREA from the file, updating the
				// user commarea length
				*calen = fread(ca, 1, *calen, statusfile);
			}
			else
			{
				// The application has no user COMMAREA, pass back zero bytes
				*calen = 0;
			}
		}
		else
		{
			// error reading header, return an error
			rc = 2;
			// Trace point for debugging
			ISAPITRACE("Error reading status file header - file not long enough");
		}

		// Close the file
		fclose(statusfile);
	}

	return rc;
}


